<?php

namespace Drupal\oauth2_rs\Server;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\oauth2_rs\AuthorizationValidators\BearerTokenValidator;
use League\OAuth2\Server\AuthorizationValidators\AuthorizationValidatorInterface;
use League\OAuth2\Server\CryptKey;
use Symfony\Bridge\PsrHttpMessage\HttpFoundationFactoryInterface;
use Symfony\Bridge\PsrHttpMessage\HttpMessageFactoryInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * OAuth2 Resource server.
 */
class ResourceServer implements ResourceServerInterface {

  /**
   * @var \League\OAuth2\Server\CryptKey
   */
  private $publicKey;

  /**
   * @var null|AuthorizationValidatorInterface
   */
  private $authorizationValidator;

  /**
   * @var \Symfony\Bridge\PsrHttpMessage\HttpMessageFactoryInterface
   */
  protected $messageFactory;

  /**
   * @var \Symfony\Bridge\PsrHttpMessage\HttpFoundationFactoryInterface
   */
  protected $foundationFactory;

  /**
   * New server instance.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   * @param \Symfony\Bridge\PsrHttpMessage\HttpMessageFactoryInterface $message_factory
   * @param \Symfony\Bridge\PsrHttpMessage\HttpFoundationFactoryInterface $foundation_factory
   * @param \League\OAuth2\Server\AuthorizationValidators\AuthorizationValidatorInterface|null $authorization_validator
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    HttpMessageFactoryInterface $message_factory,
    HttpFoundationFactoryInterface $foundation_factory,
    AuthorizationValidatorInterface $authorization_validator = NULL
  ) {
    try {
      $publicKey =  $config_factory->get('oauth2_rs.settings')
              ->get('public_key');

      $publicKeyReal = realpath($publicKey);
      if ($publicKey instanceof CryptKey === FALSE) {
        $publicKey = new CryptKey($publicKeyReal);
      }
      $this->publicKey = $publicKey;

    }
    catch (\LogicException $exception) {
      trigger_error($exception, E_USER_WARNING);
    }
    $this->messageFactory = $message_factory;
    $this->foundationFactory = $foundation_factory;

    $this->authorizationValidator = $authorization_validator;
  }

  /**
   * @return \League\OAuth2\Server\AuthorizationValidators\AuthorizationValidatorInterface
   */
  protected function getAuthorizationValidator() {
    if ($this->authorizationValidator instanceof AuthorizationValidatorInterface === FALSE) {
      $this->authorizationValidator = new BearerTokenValidator();
    }

    if ($this->authorizationValidator instanceof BearerTokenValidator === TRUE) {
      $this->authorizationValidator->setPublicKey($this->publicKey);
    }

    return $this->authorizationValidator;
  }

  /**
   * {@inheritDoc}
   */
  public function validateAuthenticatedRequest(Request $request) {
    // Create a PSR-7 message from the request that is compatible with the OAuth
    // library.
    $psr7_request = $this->messageFactory->createRequest($request);
    // Augment the request with the access token's decoded data or throw an
    // exception if authentication is unsuccessful.
    // Convert back to the Drupal/Symfony HttpFoundation objects.
    return $this->getAuthorizationValidator()->validateAuthorization($psr7_request);
  }

}
